@@ -8,6 +8,7 @@ from one_py_sdk.historian.data import HistorianApi
 from one_py_sdk.operations.spreadsheet import SpreadsheetApi
 from one_py_sdk.common.library import LibraryApi
 from one_py_sdk.enterprise.twin  import DigitalTwinApi
+from one_py_sdk.common.configuration  import ConfigurationApi
 from one_py_sdk.shared.constants import *
 from one_py_sdk.enterprise.authentication import AuthenticationApi
 from one_py_sdk.shared.helpers.csvhelper import Exporter
@@ -27,13 +28,14 @@ class ClientSdk:
 		self.Core = CoreApi(self.Environment, self.Authentication)
 		self.Historian = HistorianApi(self.Environment, self.Authentication)
 		self.Exporter = Exporter(self.Environment, self.Authentication)
+		self.Configuration = ConfigurationApi(self.Environment, self.Authentication)
 		if cacheTimeout!=None:
 			requests_cache.install_cache("client-cache", backend="memory", expire_after=cacheTimeout)
 
 	
 	def LoadCurrentUser(self):
 		if not self.Authentication.IsAuthenticated:
-			print("Not authenticated. Authenticate and try again")
+			return print("Not authenticated. Authenticate and try again")
 		if(self.Authentication.User.id != None):
 			self.Authentication.GetUserInfo()
 			self.Authentication.User.CopyFrom(self.Core.GetUser(self.Authentication.User.id))
