@@ -12,21 +12,22 @@ class HistorianApi:
         self.AppUrl = "/historian/data/v1/"
         
     def GetHistorianData(self, twinRefId, date:datetime):
-        url=f'{self.Environment}{self.AppUrl}{twinRefId}/{date}'
-        headers = {'Authorization': self.Authentication.Token.access_token, "Accept":"application/x-protobuf"}
-        print(url)
+        date =f'{str(date.date())}T{str(date.time())}Z'
+        url=f'{self.Environment}{self.AppUrl}{twinRefId}/{date}'        
+        headers = {'Authorization': self.Authentication.Token.access_token, "Accept":"application/x-protobuf"}        
         response = DeserializeResponse(requests.get(url, headers=headers))
         if response.errors:            
             return response
         return response.content.historianDatas.items
     
-    def GetHistorianDataRange(self, twinRefId, startTime:datetime, endTime: datetime):        
-        url=f'{self.Environment}{self.AppUrl}{twinRefId}?startTime={startTime}&endTime={endTime}'
-        print(url)
+    def GetHistorianDataRange(self, twinRefId, startTime:datetime, endTime: datetime):
+        startString=f'{str(startTime.date())}T{str(startTime.time())}'
+        endString=f'{str(endTime.date())}T{str(endTime.time())}'
+        url=f'{self.Environment}{self.AppUrl}{twinRefId}?startTime={startString}&endTime={endString}'        
         headers = {'Authorization': self.Authentication.Token.access_token, "Accept":"application/x-protobuf"}
         response = DeserializeResponse(requests.get(url, headers=headers))
         if response.errors:            
             return response
         return response.content.historianDatas.items
     
-        
\ No newline at end of file
+
